/* libplot.i */

%module libplot

%{
#include <plot.h>
#include "rubyio.h"

typedef plPlotter Plotter;
typedef plPlotterParams PlotterParams;

VALUE Plotter_linedash(Plotter *self, VALUE dashes, int offset) {
  int i, n;
  int *pnt;

  n = RARRAY(dashes)->len;
  pnt = ALLOCA_N(int, n);
  for (i = 0; i < n; i++) {
	pnt[i] = NUM2INT(RARRAY(dashes)->ptr[i]);
  }
  return pl_linedash_r(self, n, pnt, offset);
}

VALUE Plotter_flinedash(Plotter *self, VALUE dashes, double offset) {
  int i, n;
  double *pnt;

  n = RARRAY(dashes)->len;
  pnt = ALLOCA_N(double, n);
  for (i = 0; i < n; i++) {
	pnt[i] = NUM2DBL(RARRAY(dashes)->ptr[i]);
  }
  return pl_flinedash_r(self, n, pnt, offset);
}

%}

/*%include typemaps.i*/

/* from typemaps.i */

%typemap(in) FILE *READ {
    OpenFile *of;
    Check_Type($input, T_FILE);
    GetOpenFile($input, of);
    rb_io_check_readable(of);
    $1 = GetReadFile(of);
    rb_read_check($1);
}

%typemap(in) FILE *WRITE {
    OpenFile *of;
    Check_Type($input, T_FILE);
    GetOpenFile($input, of);
    rb_io_check_writable(of);
    $1 = GetWriteFile(of);
}

%typemap(in) FILE *STDERR {
    OpenFile *of;
    Check_Type($input, T_FILE);
    GetOpenFile($input, of);
    rb_io_check_writable(of);
    $1 = GetWriteFile(of);
}


/*
 * 
 * 
 */

typedef struct {
  %extend {
	PlotterParams() {
	  pl_newplparams();
	}
	~PlotterParams() {
	  pl_deleteplparams(self);
	}
	//plPlotterParams * pl_copyplparams ___P((const plPlotterParams *plotter_params));
	int setplparam(const char *parameter, char *value) {
	  return pl_setplparam(self, parameter, value);
	}
  }
} PlotterParams;

  
typedef struct {
  %extend {
	Plotter(char *type, FILE *READ, FILE *WRITE, FILE *STDERR, PlotterParams *plotter_params) {
	  pl_newpl_r(type, READ, WRITE, STDERR, plotter_params);
	}
	~Plotter() {
	  pl_deletepl_r(self);
	}
	int deletepl() {
	  return pl_deletepl_r(self);
	}

	/* 13 functions in traditional (pre-GNU) libplot */
	int arc(int xc, int yc, int x0, int y0, int x1, int y1) {
	  return pl_arc_r(self, xc, yc, x0, y0, x1, y1);
	}
	int box(int x0, int y0, int x1, int y1) {
	  return pl_box_r(self, x0, y0, x1, y1);
	}
	int circle(int x, int y, int r) {
	  return pl_circle_r(self, x, y, r);
	}
	int closepl() {
	  return pl_closepl_r(self);
	}
	int cont(int x, int y) {
	  return pl_cont_r(self, x, y);
	}
	int erase() {
	  return pl_erase_r(self);
	}
	int label(char *s) {
	  return pl_label_r(self, s);
	}
	int line(int x0, int y0, int x1, int y1) {
	  return pl_line_r(self, x0, y0, x1, y1);
	}
	int linemod(char *s) {
	  return pl_linemod_r(self, s);
	}
	int move(int x, int y) {
	  return pl_move_r(self, x, y);
	}
	int openpl() {
	  return pl_openpl_r(self);
	}
	int point(int x, int y) {
	  return pl_point_r(self, x, y);
	}
	int space(int x0, int y0, int x1, int y1) {
	  return pl_space_r(self, x0, y0, x1, y1);
	}

	/* 46 additional functions in GNU libplot, plus 1 obsolete function
	   [pl_outfile_r]. */
	int alabel(char *x_justify, char *y_justify, char *s) {
	  return pl_alabel_r(self, x_justify[0], y_justify[0], s);
	}
	int arcrel(int dxc, int dyc, int dx0, int dy0, int dx1, int dy1) {
	  return pl_arcrel_r(self, dxc, dyc, dx0, dy0, dx1, dy1);
	}
	int bezier2(int x0, int y0, int x1, int y1, int x2, int y2) {
	  return pl_bezier2_r(self, x0, y0, x1, y1, x2, y2);
	}
	int bezier2rel(int dx0, int dy0, int dx1, int dy1, int dx2, int dy2) {
	  return pl_bezier2rel_r(self, dx0, dy0, dx1, dy1, dx2, dy2);
	}
	int bezier3(int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3) {
	  return pl_bezier3_r(self, x0, y0, x1, y1, x2, y2, x3, y3);
	}
	int bezier3rel(int dx0, int dy0, int dx1, int dy1, int dx2, int dy2, int dx3, int dy3) {
	  return pl_bezier3rel_r(self, dx0, dy0, dx1, dy1, dx2, dy2, dx3, dy3);
	}
	int bgcolor(int red, int green, int blue) {
	  return pl_bgcolor_r(self, red, green, blue);
	}
	int bgcolorname(char *name) {
	  return pl_bgcolorname_r(self, name);
	}
	int boxrel(int dx0, int dy0, int dx1, int dy1) {
	  return pl_boxrel_r(self, dx0, dy0, dx1, dy1);
	}
	int capmod(char *s) {
	  return pl_capmod_r(self, s);
	}
	int circlerel(int dx, int dy, int r) {
	  return pl_circlerel_r(self, dx, dy, r);
	}
	int closepath() {
	  return pl_closepath_r(self);
	}
	int color(int red, int green, int blue) {
	  return pl_color_r(self, red, green, blue);
	}
	int colorname(char *name) {
	  return pl_colorname_r(self, name);
	}
	int contrel(int x, int y) {
	  return pl_contrel_r(self, x, y);
	}
	int ellarc(int xc, int yc, int x0, int y0, int x1, int y1) {
	  return pl_ellarc_r(self, xc, yc, x0, y0, x1, y1);
	}
	int ellarcrel(int dxc, int dyc, int dx0, int dy0, int dx1, int dy1) {
	  return pl_ellarcrel_r(self, dxc, dyc, dx0, dy0, dx1, dy1);
	}
	int ellipse(int x, int y, int rx, int ry, int angle) {
	  return pl_ellipse_r(self, x, y, rx, ry, angle);
	}
	int ellipserel(int dx, int dy, int rx, int ry, int angle) {
	  return pl_ellipserel_r(self, dx, dy, rx, ry, angle);
	}
	int endpath() {
	  return pl_endpath_r(self);
	}
	int endsubpath() {
	  return pl_endsubpath_r(self);
	}
	int fillcolor(int red, int green, int blue) {
	  return pl_fillcolor_r(self, red, green, blue);
	}
	int fillcolorname(char *name) {
	  return pl_fillcolorname_r(self, name);
	}
	int fillmod(char *s) {
	  return pl_fillmod_r(self, s);
	}
	int filltype(int level) {
	  return pl_filltype_r(self, level);
	}
	int flushpl() {
	  return pl_flushpl_r(self);
	}
	int fontname(char *s) {
	  return pl_fontname_r(self, s);
	}
	int fontsize(int size) {
	  return pl_fontsize_r(self, size);
	}
	int havecap(char *s) {
	  return pl_havecap_r(self, s);
	}
	int joinmod(char *s) {
	  return pl_joinmod_r(self, s);
	}
	int labelwidth(char *s) {
	  return pl_labelwidth_r(self, s);
	}
	int linedash(VALUE dashes, int offset);
	int linerel(int dx0, int dy0, int dx1, int dy1) {
	  return pl_linerel_r(self, dx0, dy0, dx1, dy1);
	}
	int linewidth(int size) {
	  return pl_linewidth_r(self, size);
	}
	int marker(int x, int y, int type, int size) {
	  return pl_marker_r(self, x, y, type, size);
	}
	int markerrel(int dx, int dy, int type, int size) {
	  return pl_markerrel_r(self, dx, dy, type, size);
	}
	int moverel(int x, int y) {
	  return pl_moverel_r(self, x, y);
	}
	int orientation(int direction) {
	  return pl_orientation_r(self, direction);
	}
	int pencolor(int red, int green, int blue) {
	  return pl_pencolor_r(self, red, green, blue);
	}
	int pencolorname(char *name) {
	  return pl_pencolorname_r(self, name);
	}
	int pentype(int level) {
	  return pl_pentype_r(self, level);
	}
	int pointrel(int dx, int dy) {
	  return pl_pointrel_r(self, dx, dy);
	}
	int restorestate() {
	  return pl_restorestate_r(self);
	}
	int savestate() {
	  return pl_savestate_r(self);
	}
	int space2(int x0, int y0, int x1, int y1, int x2, int y2) {
	  return pl_space2_r(self, x0, y0, x1, y1, x2, y2);
	}
	int textangle(int angle) {
	  return pl_textangle_r(self, angle);
	}

	/* 32 floating point counterparts to some of the above (all GNU additions) */
	double ffontname(char *s) {
	  return pl_ffontname_r(self, s);
	}
	double ffontsize(double size) {
	  return pl_ffontsize_r(self, size);
	}
	double flabelwidth(char *s) {
	  return pl_flabelwidth_r(self, s);
	}
	double ftextangle(double angle) {
	  return pl_ftextangle_r(self, angle);
	}
	int farc(double xc, double yc, double x0, double y0, double x1, double y1) {
	  pl_farc_r(self, xc, yc, x0, y0, x1, y1);
	}
	int farcrel(double dxc, double dyc, double dx0, double dy0, double dx1, double dy1) {
	  return pl_farcrel_r(self, dxc, dyc, dx0, dy0, dx1, dy1);
	}
	int fbezier2(double x0, double y0, double x1, double y1, double x2, double y2) {
	  return pl_fbezier2_r(self, x0, y0, x1, y1, x2, y2);
	}
	int fbezier2rel(double dx0, double dy0, double dx1, double dy1, double dx2, double dy2) {
	  return pl_fbezier2rel_r(self, dx0, dy0, dx1, dy1, dx2, dy2);
	}
	int fbezier3(double x0, double y0, double x1, double y1, double x2, double y2, double x3, double y3) {
	  return pl_fbezier3_r(self, x0, y0, x1, y1, x2, y2, x3, y3);
	}
	int fbezier3rel(double dx0, double dy0, double dx1, double dy1, double dx2, double dy2, double dx3, double dy3) {
	  return pl_fbezier3rel_r(self, dx0, dy0, dx1, dy1, dx2, dy2, dx3, dy3);
	}
	int fbox(double x0, double y0, double x1, double y1) {
	  return pl_fbox_r(self, x0, y0, x1, y1);
	}
	int fboxrel(double dx0, double dy0, double dx1, double dy1) {
	  return pl_fboxrel_r(self, dx0, dy0, dx1, dy1);
	}
	int fcircle(double x, double y, double r) {
	  return pl_fcircle_r(self, x, y, r);
	}
	int fcirclerel(double dx, double dy, double r) {
	  return pl_fcirclerel_r(self, dx, dy, r);
	}
	int fcont(double x, double y) {
	  return pl_fcont_r(self, x, y);
	}
	int fcontrel(double dx, double dy) {
	  return pl_fcontrel_r(self, dx, dy);
	}
	int fellarc(double xc, double yc, double x0, double y0, double x1, double y1) {
	  return pl_fellarc_r(self, xc, yc, x0, y0, x1, y1);
	}
	int fellarcrel(double dxc, double dyc, double dx0, double dy0, double dx1, double dy1) {
	  return pl_fellarcrel_r(self, dxc, dyc, dx0, dy0, dx1, dy1);
	}
	int fellipse(double x, double y, double rx, double ry, double angle) {
	  return pl_fellipse_r(self, x, y, rx, ry, angle);
	}
	int fellipserel(double dx, double dy, double rx, double ry, double angle) {
	  return pl_fellipserel_r(self, dx, dy, rx, ry, angle);
	}
	int flinedash(VALUE dashes, double offset);
	int fline(double x0, double y0, double x1, double y1) {
	  return pl_fline_r(self, x0, y0, x1, y1);
	}
	int flinerel(double dx0, double dy0, double dx1, double dy1) {
	  return pl_flinerel_r(self, dx0, dy0, dx1, dy1);
	}
	int flinewidth(double size) {
	  return pl_flinewidth_r(self, size);
	}
	int fmarker(double x, double y, int type, double size) {
	  return pl_fmarker_r(self, x, y, type, size);
	}
	int fmarkerrel(double dx, double dy, int type, double size) {
	  return pl_fmarkerrel_r(self, dx, dy, type, size);
	}
	int fmove(double x, double y) {
	  return pl_fmove_r(self, x, y);
	}
	int fmoverel(double dx, double dy) {
	  return pl_fmoverel_r(self, dx, dy);
	}
	int fpoint(double x, double y) {
	  return pl_fpoint_r(self, x, y);
	}
	int fpointrel(double dx, double dy) {
	  return pl_fpointrel_r(self, dx, dy);
	}
	int fspace(double x0, double y0, double x1, double y1) {
	  return pl_fspace_r(self, x0, y0, x1, y1);
	}
	int fspace2(double x0, double y0, double x1, double y1, double x2, double y2) {
	  return pl_fspace2_r(self, x0, y0, x1, y1, x2, y2);
	}
	
	/* 6 floating point operations with no integer counterpart (GNU additions) */
	int fconcat(double m0, double m1, double m2, double m3, double m4, double m5) {
	  return pl_fconcat_r(self, m0, m1, m2, m3, m4, m5);
	}
	int fmiterlimit(double limit) {
	  return pl_fmiterlimit_r(self, limit);
	}
	int frotate(double theta) {
	  return pl_frotate_r(self, theta);
	}
	int fscale(double x, double y) {
	  return pl_fscale_r(self, x, y);
	}
	int fsetmatrix(double m0, double m1, double m2, double m3, double m4, double m5) {
	  return pl_fsetmatrix_r(self, m0, m1, m2, m3, m4, m5);
	}
	int ftranslate(double x, double y) {
	  return pl_ftranslate_r(self, x, y);
	}

  }
} Plotter;
